top_level = [
    "math",
]

tokens = [
    "mi",
    "mn",
    "mo",
    "ms",
    "mspace",
    "mtext",
]

layout = [
    "menclose",
    "merror",
    "mfenced",
    "mfrac",
    "mpadded",
    "mphantom",
    "mroot",
    "mrow",
    "msqrt",
    "mstyle",
]

script_limit = [
    "mmultiscripts",
    "mover",
    "mprescripts",
    "msub",
    "msubsup",
    "msup",
    "munder",
    "munderover",
    "none",
]

tabular = [
    "maligngroup",
    "malignmark",
    "mtable",
    "mtd",
    "mtr",
]

elementary = [
    "mlongdiv",
    "mscarries",
    "mscarry",
    "msgroup",
    "msline",
    "msrow",
    "mstack",
]

uncategorized = [
    "maction",
]

semantic = [
    "annotation",
    "annotation-xml",
    "semantics",
]


node = [
    "mi",
    "mn",
    "mo",
    "ms",
    "mtext",
]

collection = [
    "math",
    "menclose",
    "merror",
    "mfenced",
    "mfrac",
    "mpadded",
    "mphantom",
    "mroot",
    "mrow",
    "msqrt",
    "mstyle",
    "mmultiscripts",
    "mover",
    "msub",
    "msubsup",
    "msup",
    "munder",
    "munderover",
    "maligngroup",
    "malignmark",
    "mtable",
    "mtd",
    "mtr",
]

empty = [
    "none",
    "mprescripts",
    "mspace",
]
